""" The pdelab specific parts of the code generation process """

# Trigger some imports that are needed to have all backend implementations visible
# to the selection mechanisms
from dune.perftool.pdelab.argument import (pymbolic_apply_function,
                                           pymbolic_apply_function_gradient,
                                           pymbolic_trialfunction,
                                           pymbolic_trialfunction_gradient,
                                           )
from dune.perftool.pdelab.basis import (name_basis,
                                        name_reference_gradient,
                                        )
from dune.perftool.pdelab.geometry import (dimension_iname,
                                           name_facet_jacobian_determinant,
                                           name_jacobian_determinant,
                                           name_jacobian_inverse_transposed,
                                           name_unit_inner_normal,
                                           name_unit_outer_normal,
                                           )
from dune.perftool.pdelab.index import (name_index,
                                        )
from dune.perftool.pdelab.parameter import (cell_parameter_function,
                                            intersection_parameter_function,
                                            )
from dune.perftool.pdelab.quadrature import (name_quadrature_weights,
                                             quadrature_iname,
                                             )
from dune.perftool.pdelab.spaces import (lfs_iname,
                                         )


class PDELabInterface(object):
    #
    # TODO: The following ones are actually entirely PDELab independent!
    # They should be placed elsewhere and be used directly in the visitor.
    #

    def dimension_iname(self, context=None, count=None):
        return dimension_iname(context=context, count=count)

    def name_index(self, ind):
        return name_index(ind)

    #
    # Local function space related generator functions
    #

    def lfs_iname(self, element, restriction, number):
        return lfs_iname(element, restriction, number)

    #
    # Test and trial function related generator functions
    #

    def name_basis(self, element, restriction):
        return name_basis(element, restriction)

    def name_reference_gradient(self, element, restriction):
        return name_reference_gradient(element, restriction)

    def pymbolic_trialfunction_gradient(self, element, restriction, component):
        return pymbolic_trialfunction_gradient(element, restriction, component)

    def pymbolic_apply_function_gradient(self, element, restriction, component):
        return pymbolic_apply_function_gradient(element, restriction, component)

    def pymbolic_trialfunction(self, element, restriction, component):
        return pymbolic_trialfunction(element, restriction, component)

    def pymbolic_apply_function(self, element, restriction, component):
        return pymbolic_apply_function(element, restriction, component)

    #
    # Parameter function related generator functions
    #

    def intersection_parameter_function(self, name, expr, cellwise_constant):
        return intersection_parameter_function(name, expr, cellwise_constant)

    def cell_parameter_function(self, name, expr, restriction, cellwise_constant):
        return cell_parameter_function(name, expr, restriction, cellwise_constant)

    #
    # Geometry related generator functions
    #

    def name_facet_jacobian_determinant(self):
        return name_facet_jacobian_determinant()

    def name_jacobian_determinant(self):
        return name_jacobian_determinant()

    def name_jacobian_inverse_transposed(self, restriction):
        return name_jacobian_inverse_transposed(restriction)

    def name_unit_inner_normal(self):
        return name_unit_inner_normal()

    def name_unit_outer_normal(self):
        return name_unit_outer_normal()

    #
    # Quadrature related generator functions
    #

    def quadrature_iname(self):
        return quadrature_iname()

    def name_quadrature_weights(self):
        return name_quadrature_weights()
